package cn.com.infosec.crypto.macs;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.Mac;
import cn.com.infosec.crypto.params.KeyParameter;
import cn.com.infosec.crypto.params.ParametersWithSBox;

public class GOST28147Mac implements Mac {
   private int blockSize = 8;
   private int macSize = 4;
   private int bufOff = 0;
   private byte[] buf = new byte[8];
   private byte[] mac = new byte[8];
   private boolean firstStep = true;
   private int[] workingKey = null;
   private byte[] S = new byte[]{9, 6, 3, 2, 8, 11, 1, 7, 10, 4, 14, 15, 12, 0, 13, 5, 3, 7, 14, 9, 8, 10, 15, 0, 5, 2, 6, 12, 11, 4, 13, 1, 14, 4, 6, 2, 11, 3, 13, 8, 12, 15, 5, 10, 0, 7, 1, 9, 14, 7, 10, 12, 13, 1, 3, 9, 0, 2, 11, 4, 15, 8, 5, 6, 11, 5, 1, 9, 8, 13, 15, 0, 14, 4, 2, 3, 12, 7, 10, 6, 3, 10, 13, 12, 1, 2, 0, 11, 7, 5, 9, 4, 8, 15, 14, 6, 1, 13, 2, 9, 7, 10, 6, 0, 8, 12, 4, 5, 15, 3, 11, 14, 11, 10, 15, 5, 0, 12, 14, 8, 6, 2, 3, 9, 1, 7, 13, 4};

   private int[] generateWorkingKey(byte[] userKey) {
      if (userKey.length != 32) {
         throw new IllegalArgumentException("Key length invalid. Key needs to be 32 byte - 256 bit!!!");
      } else {
         int[] var2 = new int[8];

         for(int var3 = 0; var3 != 8; ++var3) {
            var2[var3] = this.bytesToint(userKey, var3 * 4);
         }

         return var2;
      }
   }

   private int gost28147_mainStep(int n1, int key) {
      GOST28147Mac var10000 = this;
      int var3 = key + n1;
      byte[] n11;
      return (var3 = ((n11 = var10000.S)[(var3 >> 0 & 15) + 0] << 0) + (n11[(var3 >> 4 & 15) + 16] << 4) + (n11[(var3 >> 8 & 15) + 32] << 8) + (n11[(var3 >> 12 & 15) + 48] << 12) + (n11[(var3 >> 16 & 15) + 64] << 16) + (n11[(var3 >> 20 & 15) + 80] << 20) + (n11[(var3 >> 24 & 15) + 96] << 24) + (n11[(var3 >> 28 & 15) + 112] << 28)) << 11 | var3 >>> 21;
   }

   private void gost28147MacFunc(int[] workingKey, byte[] in, int inOff, byte[] out, int outOff) {
      int var6 = this.bytesToint(in, inOff);
      int in1 = this.bytesToint(in, inOff + 4);

      int var10000;
      for(inOff = 0; inOff < 2; ++inOff) {
         for(int var7 = 0; var7 < 8; var6 = var10000) {
            in1 ^= this.gost28147_mainStep(var6, workingKey[var7]);
            ++var7;
            var10000 = in1;
            in1 = var6;
         }
      }

      this.intTobytes(var6, out, outOff);
      int var8 = outOff + 4;
      this.intTobytes(in1, out, var8);
   }

   private int bytesToint(byte[] in, int inOff) {
      return (in[inOff + 3] << 24 & -16777216) + (in[inOff + 2] << 16 & 16711680) + (in[inOff + 1] << 8 & '\uff00') + (in[inOff] & 255);
   }

   private void intTobytes(int num, byte[] out, int outOff) {
      int var4 = outOff + 3;
      out[var4] = (byte)(num >>> 24);
      var4 = outOff + 2;
      out[var4] = (byte)(num >>> 16);
      var4 = outOff + 1;
      out[var4] = (byte)(num >>> 8);
      out[outOff] = (byte)num;
   }

   private byte[] CM5func(byte[] buf, int bufOff, byte[] mac) {
      byte[] var4 = new byte[buf.length - bufOff];
      int buf1 = mac.length;
      System.arraycopy(buf, bufOff, var4, 0, buf1);

      for(buf1 = 0; buf1 != mac.length; ++buf1) {
         var4[buf1] ^= mac[buf1];
      }

      return var4;
   }

   public void init(CipherParameters params) {
      this.reset();
      this.buf = new byte[this.blockSize];
      if (params instanceof ParametersWithSBox) {
         ParametersWithSBox var10000 = (ParametersWithSBox)params;
         ParametersWithSBox params1;
         byte[] var10001 = (params1 = (ParametersWithSBox)params).getSBox();
         byte[] var2 = this.S;
         int var3 = params1.getSBox().length;
         System.arraycopy(var10001, 0, var2, 0, var3);
         if (var10000.getParameters() != null) {
            this.workingKey = this.generateWorkingKey(((KeyParameter)params1.getParameters()).getKey());
         }
      } else {
         if (!(params instanceof KeyParameter)) {
            throw new IllegalArgumentException("invalid parameter passed to GOST28147 init - " + params.getClass().getName());
         }

         this.workingKey = this.generateWorkingKey(((KeyParameter)params).getKey());
      }

   }

   public String getAlgorithmName() {
      return "GOST28147Mac";
   }

   public int getMacSize() {
      return this.macSize;
   }

   public void update(byte in) {
      byte[] var2;
      if (this.bufOff == (var2 = this.buf).length) {
         byte[] var10001 = var2;
         var2 = new byte[var2.length];
         int var3 = this.mac.length;
         System.arraycopy(var10001, 0, var2, 0, var3);
         if (this.firstStep) {
            this.firstStep = false;
         } else {
            var2 = this.buf;
            byte[] var6 = this.mac;
            var2 = this.CM5func(var2, 0, var6);
         }

         int[] var7 = this.workingKey;
         byte[] var4 = this.mac;
         this.gost28147MacFunc(var7, var2, 0, var4, 0);
         this.bufOff = 0;
      }

      byte[] var10000 = this.buf;
      int var5;
      this.bufOff = (var5 = this.bufOff) + 1;
      var10000[var5] = in;
   }

   public void update(byte[] in, int inOff, int len) {
      if (len < 0) {
         throw new IllegalArgumentException("Can't have a negative input length!");
      } else {
         int var4;
         int var5;
         int var10000;
         if (len > (var5 = this.blockSize - (var4 = this.bufOff))) {
            System.arraycopy(in, inOff, this.buf, var4, var5);
            byte[] var10001 = this.buf;
            byte[] var12 = new byte[var10001.length];
            int var6 = this.mac.length;
            System.arraycopy(var10001, 0, var12, 0, var6);
            if (this.firstStep) {
               this.firstStep = false;
            } else {
               var12 = this.buf;
               byte[] var13 = this.mac;
               var12 = this.CM5func(var12, 0, var13);
            }

            var10000 = inOff;
            int var15 = len;
            int[] inOff1 = this.workingKey;
            byte[] len1 = this.mac;
            this.gost28147MacFunc(inOff1, var12, 0, len1, 0);
            this.bufOff = 0;
            inOff = var15 - var5;

            for(len = var10000 + var5; inOff > this.blockSize; inOff = var10000) {
               var10000 = len;
               var15 = inOff;
               byte[] inOff2 = this.mac;
               inOff2 = this.CM5func(in, len, inOff2);
               int[] len2 = this.workingKey;
               var12 = this.mac;
               this.gost28147MacFunc(len2, inOff2, 0, var12, 0);
               len = var15 - (inOff = this.blockSize);
               inOff += var10000;
               var10000 = len;
               len = inOff;
            }
         } else {
            var10000 = inOff;
            inOff = len;
            len = var10000;
         }

         GOST28147Mac var14 = this;
         GOST28147Mac var16 = this;
         GOST28147Mac var10004 = this;
         byte[] var7 = this.buf;
         System.arraycopy(in, len, var7, var10004.bufOff, inOff);
         var14.bufOff = var16.bufOff + inOff;
      }
   }

   public int doFinal(byte[] out, int outOff) {
      int var3;
      while((var3 = this.bufOff) < this.blockSize) {
         this.buf[var3] = 0;
         this.bufOff = var3 + 1;
      }

      byte[] var10001 = this.buf;
      byte[] var7 = new byte[var10001.length];
      int var4 = this.mac.length;
      System.arraycopy(var10001, 0, var7, 0, var4);
      if (this.firstStep) {
         this.firstStep = false;
      } else {
         var7 = this.buf;
         byte[] var8 = this.mac;
         var7 = this.CM5func(var7, 0, var8);
      }

      GOST28147Mac var10000 = this;
      GOST28147Mac var10 = this;
      int[] var9 = this.workingKey;
      byte[] var5 = this.mac;
      this.gost28147MacFunc(var9, var7, 0, var5, 0);
      byte[] var10002 = this.mac;
      int var6;
      System.arraycopy(var10002, var10002.length / 2 - (var6 = this.macSize), out, outOff, var6);
      var10.reset();
      return var10000.macSize;
   }

   public void reset() {
      byte[] var2;
      for(int var1 = 0; var1 < (var2 = this.buf).length; ++var1) {
         var2[var1] = 0;
      }

      this.bufOff = 0;
      this.firstStep = true;
   }
}
